%undefine _ld_as_needed
%global majorver %%(echo %{version} | cut -d. -f1-2)

Summary: Single precision real SuperLU routines for shared memory parallel machines
Name:    SuperLUMT
Version: 3.1.0
Release: 2%{?dist}
License: BSD
URL:     http://crd-legacy.lbl.gov/~xiaoye/SuperLU
Source0: %{url}/superlu_mt_%{majorver}.tar.gz

BuildRequires: make pkgconfig(flexiblas) pkgconfig gcc

Patch3000: %{name}-build_shared.patch
Patch3001: %{name}-fix_testsuite.patch
Patch3002: %{name}64-build_shared.patch
Patch3003: %{name}64-fix_testsuite.patch
Patch3004: %{name}-fix_examples.patch
Patch3005: %{name}64-fix_examples.patch
Patch3006: SuperLUMT-c99-1.patch
Patch3007: SuperLUMT-c99-2.patch
Patch3008: SuperLUMT-c99-3.patch

Requires: %{name}-common = %{version}-%{release}

%description
Subroutines to solve sparse linear systems for shared memory parallel machines.  SuperLU contains a
set of subroutines to solve a sparse linear system A*X=B. It uses Gaussian elimination with partial
pivoting (GEPP).  The columns of A may be preordered before factorization; the preordering for
sparsity is completely separate from the factorization.

%package double
Summary: Double precision real SuperLU routines for shared memory parallel machines
Requires: %{name}-common = %{version}-%{release}

%description double
This package contains double precision real SuperLU routines library by SuperLUMT.

%package complex
Summary: Single precision complex SuperLU routines for shared memory parallel machines
Requires: %{name}-common = %{version}-%{release}

%description complex
This package contains single precision complex routines library by SuperLUMT.

%package complex16
Summary: Double precision complex SuperLU routines for shared memory parallel machines
Requires: %{name}-common = %{version}-%{release}

%description complex16
This package contains double precision complex routines library by SuperLUMT.

%package devel
Summary: The SuperLUMT headers and development-related files
Requires: %{name} = %{version}-%{release}
Requires: %{name}-double = %{version}-%{release}
Requires: %{name}-complex = %{version}-%{release}
Requires: %{name}-complex16 = %{version}-%{release}

%description devel
Shared links and header files used by SuperLUMT.

%package -n SuperLUMT64
Summary: Single precision real SuperLU routines (64bit INTEGER)
BuildRequires: pkgconfig(flexiblas)
Requires: %{name}-common = %{version}-%{release}

%description -n SuperLUMT64
Subroutines to solve sparse linear systems for shared memory parallel machines (64bit INTEGER).
SuperLU contains a set of subroutines to solve a sparse linear system A*X=B. It uses Gaussian
elimination with partial pivoting (GEPP).  The columns of A may be preordered before factorization;
the preordering for sparsity is completely separate from the factorization.

%package -n SuperLUMT64-double
Summary: Double precision real SuperLU routines (64bit INTEGER)
Requires: %{name}-common = %{version}-%{release}

%description -n SuperLUMT64-double
This package contains double precision real SuperLU routines library by SuperLUMT (64bit INTEGER).

%package -n SuperLUMT64-complex
Summary: Single precision complex SuperLU routines (64bit INTEGER)
Requires: %{name}-common = %{version}-%{release}

%description -n SuperLUMT64-complex
This package contains single precision complex routines library by SuperLUMT (64bit INTEGER).

%package -n SuperLUMT64-complex16
Summary: Double precision complex SuperLU routines (64bit INTEGER)
Requires: %{name}-common = %{version}-%{release}

%description -n SuperLUMT64-complex16
This package contains double precision complex routines library by SuperLUMT (64bit INTEGER).

%package -n SuperLUMT64-devel
Summary: The MUMPS headers and development-related files (64bit INTEGER)
Requires: SuperLUMT64 = %{version}-%{release}
Requires: SuperLUMT64-double = %{version}-%{release}
Requires: SuperLUMT64-complex = %{version}-%{release}
Requires: SuperLUMT64-complex16 = %{version}-%{release}

%description -n SuperLUMT64-devel
Shared links, header files for %{name} (64bit INTEGER).

%package common
Summary: Documentation files for SuperLUMT
BuildArch: noarch

%description common
This package contains common documentation files for SuperLUMT.

%prep
%setup -q -n SuperLU_MT_%{majorver}

rm -fr SRC/mc64ad.f.bak
find . -type f | sed -e "/TESTING/d" | xargs chmod a-x
find EXAMPLE -type f | while read file
do
   [ "$(file $file | awk '{print $2}')" = ELF ] && rm $file || :
done

cp -a EXAMPLE EXAMPLE64

pushd EXAMPLE64
%patch3007 -p2
popd

%patch3000 -p0
%patch3001 -p0
%patch3004 -p0
%patch3006 -p1
%patch3007 -p1
%patch3008 -p1

%build
cp -p MAKE_INC/make.linux.openmp make.inc
sed -i -e "s|-O3|$RPM_OPT_FLAGS|" \
make.inc

export LIBBLASLINK=-lflexiblas
export LDFLAGS="%{__global_ldflags} -lgomp $LIBBLASLINK" \

make %{?_smp_mflags} \
 SONAME=%{majorver} \
 BLASLIB="-L%{_libdir} $LIBBLASLINK" \
 PREDEFS="$RPM_OPT_FLAGS $LDFLAGS -fPIC -D__OPENMP -DPRNTlevel=0 -DDEBUGlevel=0" \
 NOOPTS="-O0 -fPIC -fopenmp $LIBBLASLINK" \
 CDEFS=-DAdd_ \
 FFLAGS="$RPM_OPT_FLAGS $LDFLAGS -fPIC -fopenmp -D__OPENMP -DPRNTlevel=0 -DDEBUGlevel=0" \
 FORTRAN=gfortran \
 LOADER=gcc \
 CC=gcc \
 CFLAGS="$RPM_OPT_FLAGS $LDFLAGS -fPIC -D__OPENMP -DPRNTlevel=0 -DDEBUGlevel=0" \
 LDFLAGS="%{__global_ldflags} -lgomp $LIBBLASLINK" \
 MATHLIB=-lm \
 MPLIB= -C SRC single double complex complex16

cp -p SRC/libsuperlumt_*.so.%{majorver} lib/
cp -p SRC/libsuperlumt_*.so lib/

export LIBBLASLINK=-lflexiblas
export LDFLAGS="%{__global_ldflags} -lgomp $LIBBLASLINK" \

make %{?_smp_mflags} \
 SONAME=%{majorver} \
 BLASLIB="-L%{_libdir} $LIBBLASLINK" \
 PREDEFS="$RPM_OPT_FLAGS $LDFLAGS -fPIC -D__OPENMP -DPRNTlevel=0 -DDEBUGlevel=0" \
 NOOPTS="-O0 -fPIC -fopenmp $LIBBLASLINK" \
 CDEFS=-DAdd_ \
 FFLAGS="$RPM_OPT_FLAGS $LDFLAGS -fPIC -fopenmp -D__OPENMP -DPRNTlevel=0 -DDEBUGlevel=0" \
 FORTRAN=gfortran \
 LOADER=gcc \
 LOADOPTS="$RPM_OPT_FLAGS $LDFLAGS -fPIC -D__OPENMP -DPRNTlevel=0 -DDEBUGlevel=0" \
 CC=gcc \
 CFLAGS="$RPM_OPT_FLAGS $LDFLAGS -fPIC -D__OPENMP -DPRNTlevel=0 -DDEBUGlevel=0" \
 LDFLAGS="%{__global_ldflags} -lgomp $LIBBLASLINK" \
 MATHLIB=-lm \
 MPLIB= -C EXAMPLE single double complex complex16

make -C SRC clean
make -C TESTING/MATGEN clean
patch -R -p0 < %{PATCH3000}
patch -R -p0 < %{PATCH3001}
patch -p0 < %{PATCH3002}
patch -p0 < %{PATCH3003}
patch -p0 < %{PATCH3005}

export LIBBLASLINK=-lflexiblas64
export LDFLAGS="%{__global_ldflags} -lgomp $LIBBLASLINK" \

make %{?_smp_mflags} \
 SONAME=%{majorver} \
 BLASLIB="-L%{_libdir} $LIBBLASLINK" \
 PREDEFS="$RPM_OPT_FLAGS $LDFLAGS -fPIC -D__OPENMP -DPRNTlevel=0 -DDEBUGlevel=0 -D_LONGINT" \
 NOOPTS="-O0 -fPIC -fopenmp $LIBBLASLINK" \
 CDEFS=-DAdd_ \
 FFLAGS="$RPM_OPT_FLAGS $LDFLAGS -fPIC -fopenmp -D__OPENMP -DPRNTlevel=0 -DDEBUGlevel=0 -fdefault-integer-8" \
 FORTRAN=gfortran \
 CFLAGS="$RPM_OPT_FLAGS $LDFLAGS -fPIC -D__OPENMP -DPRNTlevel=0 -DDEBUGlevel=0 -D_LONGINT" \
 LOADER=gcc \
 CC=gcc \
 LDFLAGS="%{__global_ldflags} -lgomp $LIBBLASLINK" \
 MATHLIB=-lm \
 MPLIB= -C SRC single double complex complex16

cp -p SRC/libsuperlumt64_*.so.%{majorver} lib
cp -p SRC/libsuperlumt64_*.so lib


export LIBBLASLINK=-lflexiblas64
export LDFLAGS="%{__global_ldflags} -lgomp $LIBBLASLINK" \

make %{?_smp_mflags} \
 SONAME=%{majorver} \
 BLASLIB="-L%{_libdir} $LIBBLASLINK" \
 PREDEFS="$RPM_OPT_FLAGS $LDFLAGS -fPIC -D__OPENMP -DPRNTlevel=0 -DDEBUGlevel=0 -D_LONGINT" \
 NOOPTS="-O0 -fPIC -fopenmp $LIBBLASLINK" \
 CDEFS=-DAdd_ \
 FFLAGS="$RPM_OPT_FLAGS $LDFLAGS -fPIC -fopenmp -D__OPENMP -DPRNTlevel=0 -DDEBUGlevel=0 -fdefault-integer-8" \
 FORTRAN=gfortran \
 CFLAGS="$RPM_OPT_FLAGS $LDFLAGS -fPIC -D__OPENMP -DPRNTlevel=0 -DDEBUGlevel=0 -D_LONGINT" \
 LOADER=gcc \
 LOADOPTS="$RPM_OPT_FLAGS $LDFLAGS -fPIC -D__OPENMP -DPRNTlevel=0 -DDEBUGlevel=0 -D_LONGINT" \
 CC=gcc \
 LDFLAGS="%{__global_ldflags} -lgomp $LIBBLASLINK" \
 MATHLIB=-lm \
 MPLIB= -C EXAMPLE64 single double complex complex16


%check
pushd EXAMPLE
export LD_LIBRARY_PATH=%{buildroot}%{_libdir}:%{_libdir}
./pslinsol < big.rua
./pdlinsol < big.rua
./pclinsol < cmat
./pzlinsol < cmat
./pslinsolx < big.rua
./pdlinsolx < big.rua
./pclinsolx < cmat
./pzlinsolx < cmat
./pslinsolx1 < big.rua
./pdlinsolx1 < big.rua
./pclinsolx1 < cmat
./pzlinsolx1 < cmat
popd

pushd EXAMPLE64
export LD_LIBRARY_PATH=%{buildroot}%{_libdir}:%{_libdir}
./pslinsol < big.rua
./pdlinsol < big.rua
./pclinsol < cmat
./pzlinsol < cmat
popd

%install
mkdir -p %{buildroot}%{_libdir}
mkdir -p %{buildroot}%{_includedir}/%{name}
cp -P lib/libsuperlumt_*.so.%{majorver} %{buildroot}%{_libdir}
install -p SRC/*.h %{buildroot}%{_includedir}/%{name}
chmod -x %{buildroot}%{_includedir}/%{name}/*.h
cp -P lib/libsuperlumt_*.so %{buildroot}%{_libdir}

for i in s d c z
do
 ln -sf  %{_libdir}/libsuperlumt_${i}.so.%{majorver} %{buildroot}%{_libdir}/libsuperlumt_${i}.so
done

cp -P lib/libsuperlumt64_*.so.%{majorver} %{buildroot}%{_libdir}
cp -P lib/libsuperlumt64_*.so %{buildroot}%{_libdir}

for i in s d c z
do
 ln -sf  %{_libdir}/libsuperlumt64_${i}.so.%{majorver} %{buildroot}%{_libdir}/libsuperlumt64_${i}.so
done

%files
%{_libdir}/libsuperlumt_s.so.%{majorver}

%files double
%{_libdir}/libsuperlumt_d.so.%{majorver}

%files complex
%{_libdir}/libsuperlumt_c.so.%{majorver}

%files complex16
%{_libdir}/libsuperlumt_z.so.%{majorver}

%files devel
%{_includedir}/%{name}/
%{_libdir}/libsuperlumt_*.so

%files -n SuperLUMT64
%{_libdir}/libsuperlumt64_s.so.%{majorver}

%files -n SuperLUMT64-double
%{_libdir}/libsuperlumt64_d.so.%{majorver}

%files -n SuperLUMT64-complex
%{_libdir}/libsuperlumt64_c.so.%{majorver}

%files -n SuperLUMT64-complex16
%{_libdir}/libsuperlumt64_z.so.%{majorver}

%files -n SuperLUMT64-devel
%{_includedir}/%{name}/
%{_libdir}/libsuperlumt64_*.so

%files common
%license License.txt
%doc DOC README

%changelog
* Fri Apr 28 2023 OpenCloudOS Release Engineering <releng@opencloudos.tech> - 3.1.0-2
- Rebuilt for OpenCloudOS Stream 23.05

* Tue Apr 25 2023 Shuo Wang <abushwang@tencent.com> - 3.1.0-1
- initial build
